package main

import (
	"database/sql"
	"encoding/json"
	"fmt"
	_ "github.com/lib/pq"
	"log"
	"net/http"
	"strconv"
	"strings"
)

const (
	dburl = "postgres://go-http-server:h4l10@localhost/go-http-server"
	dbstr = "postgres"
)

type MeasurementHandler struct {
	db *sql.DB
}

type MeasurementData struct {
	Sensor1 int64   `json:"sensor1"`
	Sensor2 int64   `json:"sensor2"`
	Sensor3 float64 `json:"sensor3"`
}

type CalibrationHandler struct {
	db *sql.DB
}

type CalibrationData struct {
	Value1 int     `json:"value1"`
	Value2 float64 `json:"value2"`
}

func (c *CalibrationHandler) Serve(w http.ResponseWriter, r *http.Request) {
	dbid, err := strconv.ParseInt(strings.TrimPrefix(r.RequestURI, "/calibration/"), 10, 64)
	if err != nil {
		Http500(w, "Could not parse database id")
		return
	}
	cd := CalibrationData{}
	err = c.db.QueryRow("SELECT value1, value2 FROM calibration WHERE id=$1", dbid).Scan(&cd.Value1, &cd.Value2)
	if err != nil {
		if err == sql.ErrNoRows {
			Http404(w)
			return
		}
		Http500(w, "Could not execute db query")
		return
	}
	rv, err := json.Marshal(cd)
	if err != nil {
		Http500(w, "Could not marshal data")
		return
	}
	w.Header().Set("Content-Type", "application/json")
	fmt.Fprintf(w, "%s", string(rv))
}

func (m MeasurementHandler) Serve(w http.ResponseWriter, r *http.Request) {
	var measurement MeasurementData
	err := json.NewDecoder(r.Body).Decode(&measurement)
	if err != nil {
		Http500(w, "Could not decode request body")
		return
	}
	var id int
	err = m.db.QueryRow("INSERT INTO measurement(sensor1, sensor2, sensor3) VALUES ($1, $2, $3) RETURNING id",
		measurement.Sensor1, measurement.Sensor2, measurement.Sensor3).Scan(&id)
	if err != nil {
		w.Header().Set("Content-Type", "application/json")
		w.WriteHeader(http.StatusNotFound)
		errmsg := map[string]string{"error": "Not written"}
		rv, err := json.Marshal(errmsg)
		if err != nil {
			Http500(w, "Could not marshal data")
			return
		}
		w.Write(rv)
	}
	return
}

func Http500(w http.ResponseWriter, message string) {
	errmsg := map[string]string{"error": message}
	rv, err := json.Marshal(errmsg)
	if err != nil {
		Http500(w, "Could not marshal")
		return
	}
	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusInternalServerError)
	w.Write(rv)
	return
}

func Http404(w http.ResponseWriter) {
	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusNotFound)
	errmsg := map[string]string{"error": "Sorry, not found"}
	rv, err := json.Marshal(errmsg)
	if err != nil {
		Http500(w, "Could not marshal data")
		return
	}
	w.Write(rv)
	return
}

func main() {
	db, err := sql.Open(dbstr, dburl)
	defer db.Close()
	if err != nil {
		panic(err)
	}
	c := CalibrationHandler{db: db}
	m := MeasurementHandler{db: db}
	http.HandleFunc("/calibration/", c.Serve)
	http.HandleFunc("/measurement/", m.Serve)
	log.Fatal(http.ListenAndServe(":3000", nil))
}
